@@ -11,6 +11,7 @@ import pyqtgraph as pg
 
 import Orange.data
 from Orange.data import Domain, Table, ContinuousVariable
+from Orange.data.util import hstack
 import Orange.projection
 from Orange.projection.manifold import torgerson
 import Orange.distance
@@ -21,8 +22,9 @@ from Orange.widgets.utils.sql import check_sql_input
 from Orange.canvas import report
 from Orange.widgets.visualize.owscatterplotgraph import OWScatterPlotGraph, InteractiveViewBox
 from Orange.widgets.widget import Msg, OWWidget, Input, Output
-from Orange.widgets.utils.annotated_data import (create_annotated_table,
-                                                 ANNOTATED_DATA_SIGNAL_NAME, get_unique_names)
+from Orange.widgets.utils.annotated_data import (
+    ANNOTATED_DATA_SIGNAL_NAME, create_annotated_table, create_groups_table,
+    get_unique_names)
 
 
 def stress(X, distD):
@@ -42,6 +44,11 @@ class MDSInteractiveViewBox(InteractiveViewBox):
 class OWMDSGraph(OWScatterPlotGraph):
     jitter_size = settings.Setting(0)
 
+    def __init__(self, scatter_widget, parent=None, name="None", view_box=None):
+        super().__init__(scatter_widget, parent=parent, _=name, view_box=view_box)
+        for axis_loc in ["left", "bottom"]:
+            self.plot_widget.hideAxis(axis_loc)
+
     def update_data(self, attr_x, attr_y, reset_view=True):
         super().update_data(attr_x, attr_y, reset_view=reset_view)
         for axis in ["left", "bottom"]:
@@ -263,6 +270,11 @@ class OWMDS(OWWidget):
 
     def init_attr_values(self):
         domain = self.data and len(self.data) and self.data.domain or None
+        if domain is not None:
+            domain = Domain(
+                attributes=domain.attributes,
+                class_vars=domain.class_vars,
+                metas=tuple(a for a in domain.metas if a.is_primitive()))
         for model in self.models:
             model.set_domain(domain)
         self.graph.attr_color = self.data.domain.class_var if domain else None
@@ -303,6 +315,7 @@ class OWMDS(OWWidget):
         if self.matrix is not None and data is not None and len(self.matrix) == len(data):
             self.closeContext()
             self.data = data
+            self.init_attr_values()
             self.openContext(data)
         else:
             self._invalidated = True
@@ -359,6 +372,7 @@ class OWMDS(OWWidget):
         self.data = None
         self.effective_matrix = None
         self.embedding = None
+        self.init_attr_values()
 
         # if no data nor matrix is present reset plot
         if self.signal_data is None and self.matrix is None:
@@ -464,7 +478,7 @@ class OWMDS(OWWidget):
         MDS points, `stress` is the current stress and `progress` a float
         ratio (0 <= progress <= 1)
 
-        If an existing update coroutine loop is already in palace it is
+        If an existing update coroutine loop is already in place it is
         interrupted (i.e. closed).
 
         .. note::
@@ -563,10 +577,10 @@ class OWMDS(OWWidget):
 
     def handleNewSignals(self):
         if self._invalidated:
+            self.__draw_similar_pairs = False
             self._invalidated = False
             self._initialize()
             self.start()
-        self.__draw_similar_pairs = False
 
         if self._subset_mask is None and self.subset_data is not None and \
                 self.data is not None:
@@ -580,7 +594,7 @@ class OWMDS(OWWidget):
 
     def _on_connected_changed(self):
         self._similar_pairs = None
-        self._update_plot()
+        self.connect_pairs()
 
     def _update_plot(self, new=False):
         self._clear_plot()
@@ -591,6 +605,8 @@ class OWMDS(OWWidget):
             self.graph.new_data(None)
 
     def connect_pairs(self):
+        if self._curve:
+            self.graph.plot_widget.removeItem(self._curve)
         if not (self.connected_pairs and self.__draw_similar_pairs):
             return
         emb_x, emb_y = self.graph.get_xy_data_positions(
@@ -626,8 +642,6 @@ class OWMDS(OWWidget):
         pairs_mask = ~(np.isclose(x1, x2) & np.isclose(y1, y2))
         emb_x_pairs = emb_x_pairs[pairs_mask, :]
         emb_y_pairs = emb_y_pairs[pairs_mask, :]
-        if self._curve:
-            self.graph.plot_widget.removeItem(self._curve)
         self._curve = pg.PlotCurveItem(
             emb_x_pairs.ravel(), emb_y_pairs.ravel(),
             pen=pg.mkPen(0.8, width=2, cosmetic=True),
@@ -637,13 +651,23 @@ class OWMDS(OWWidget):
     def _setup_plot(self, new=False):
         emb_x, emb_y = self.embedding[:, 0], self.embedding[:, 1]
         coords = np.vstack((emb_x, emb_y)).T
-        attributes = () + self.data.domain.attributes + (self.variable_x, self.variable_y)
+
+        data = self.data
+
+        primitive_metas = tuple(a for a in data.domain.metas if a.is_primitive())
+        keys = [k for k, a in enumerate(data.domain.metas) if a.is_primitive()]
+        data_metas = data.metas[:, keys].astype(float)
+
+        attributes = self.data.domain.attributes + (self.variable_x, self.variable_y) + \
+                     primitive_metas
         domain = Domain(attributes=attributes,
-                        class_vars=self.data.domain.class_vars,
-                        metas=self.data.domain.metas)
-        data = Table.from_numpy(domain, X=np.hstack((self.data.X, coords)),
-                                Y=self.data.Y,
-                                metas=self.data.metas)
+                        class_vars=self.data.domain.class_vars)
+        if data_metas is not None:
+            data_x = (self.data.X, coords, data_metas)
+        else:
+            data_x = (self.data.X, coords)
+        data = Table.from_numpy(domain, X=hstack(data_x),
+                                Y=self.data.Y)
         subset_data = data[self._subset_mask] if self._subset_mask is not None else None
         self.graph.new_data(data, subset_data=subset_data, new=new)
         self.graph.update_data(self.variable_x, self.variable_y, True)
@@ -672,8 +696,12 @@ class OWMDS(OWWidget):
             selected = output[selection]
         else:
             selected = None
+        if self.graph.selection is not None and np.max(self.graph.selection) > 1:
+            annotated = create_groups_table(output, self.graph.selection)
+        else:
+            annotated = create_annotated_table(output, selection)
         self.Outputs.selected_data.send(selected)
-        self.Outputs.annotated_data.send(create_annotated_table(output, selection))
+        self.Outputs.annotated_data.send(annotated)
 
     def onDeleteWidget(self):
         super().onDeleteWidget()
