@@ -4,6 +4,7 @@ from xml.sax.saxutils import escape
 from math import log10, floor, ceil
 
 import numpy as np
+import scipy.sparse as sp
 from scipy.stats import linregress
 
 from AnyQt.QtCore import Qt, QObject, QEvent, QRectF, QPointF, QSize
@@ -21,7 +22,6 @@ from pyqtgraph.graphicsItems.TextItem import TextItem
 from pyqtgraph.graphicsItems.InfiniteLine import InfiniteLine
 from pyqtgraph.Point import Point
 
-
 from Orange.widgets import gui
 from Orange.widgets.utils import classdensity, get_variable_values_sorted
 from Orange.widgets.utils.colorpalette import (ColorPaletteGenerator,
@@ -470,14 +470,10 @@ _define_symbols()
 
 
 class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
-    attr_color = ContextSetting(
-        None, ContextSetting.OPTIONAL, exclude_metas=False)
-    attr_label = ContextSetting(
-        None, ContextSetting.OPTIONAL, exclude_metas=False)
-    attr_shape = ContextSetting(
-        None, ContextSetting.OPTIONAL, exclude_metas=False)
-    attr_size = ContextSetting(
-        None, ContextSetting.OPTIONAL, exclude_metas=False)
+    attr_color = ContextSetting(None, required=ContextSetting.OPTIONAL)
+    attr_label = ContextSetting(None, required=ContextSetting.OPTIONAL)
+    attr_shape = ContextSetting(None, required=ContextSetting.OPTIONAL)
+    attr_size = ContextSetting(None, required=ContextSetting.OPTIONAL)
     label_only_selected = Setting(False)
 
     point_width = Setting(10)
@@ -504,7 +500,9 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
         self.plot_widget.sizeHint = lambda: QSize(500, 500)
         scene = self.plot_widget.scene()
         self._create_drag_tooltip(scene)
-
+        self._data = None  # Original Table as passed from widget to new_data before transformations
+        self.attr_x = None
+        self.attr_y = None
 
         self.replot = self.plot_widget.replot
         ScaleScatterPlotData.__init__(self)
@@ -609,8 +607,33 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
 
         self.subset_indices = set(e.id for e in subset_data) if subset_data else None
 
+        self._data = data
+        data = self.sparse_to_dense()
         self.set_data(data, **args)
 
+    def sparse_to_dense(self):
+        data = self._data
+        if data is None or not data.is_sparse():
+            return data
+
+        attrs = {self.attr_x,
+                 self.attr_y,
+                 self.attr_color,
+                 self.attr_shape,
+                 self.attr_size,
+                 self.attr_label}
+        domain = data.domain
+        all_attrs = domain.variables + domain.metas
+        attrs = list(set(all_attrs) & attrs)
+        selected_data = data[:, attrs]
+        if sp.issparse(selected_data.X):
+            selected_data.X = selected_data.X.toarray()
+        if sp.issparse(selected_data.Y):
+            selected_data.Y = selected_data.Y.toarray()
+        if sp.issparse(selected_data.metas):
+            selected_data.metas = selected_data.metas.toarray()
+        return selected_data
+
     def _clear_plot_widget(self):
         self.remove_legend()
         if self.density_img:
@@ -632,6 +655,12 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
         self.set_axis_title("left", "")
 
     def update_data(self, attr_x, attr_y, reset_view=True):
+        self.attr_x = attr_x
+        self.attr_y = attr_y
+        if attr_x not in self.data.domain or attr_y not in self.data.domain:
+            data = self.sparse_to_dense()
+            self.set_data(data)
+
         self.master.Warning.missing_coords.clear()
         self.master.Information.missing_coords.clear()
         self._clear_plot_widget()
@@ -721,7 +750,7 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
 
     def draw_regression_line(self, x_data, y_data, min_x, max_x):
         if self.show_reg_line and self.can_draw_regresssion_line():
-            slope, intercept, _, _, _ = linregress(x_data, y_data)
+            slope, intercept, rvalue, _, _ = linregress(x_data, y_data)
             start_y = min_x * slope + intercept
             end_y = max_x * slope + intercept
             angle = np.degrees(np.arctan((end_y - start_y) / (max_x - min_x)))
@@ -731,7 +760,7 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
                           rotateAxis=(1, 0), movable=True)
             self.reg_line_item = InfiniteLine(
                 pos=QPointF(min_x, start_y), pen=pg.mkPen(color=color, width=1),
-                angle=angle, label="r = {:.2f}".format(slope), labelOpts=l_opts)
+                angle=angle, label="r = {:.2f}".format(rvalue), labelOpts=l_opts)
             if rotate:
                 self.reg_line_item.label.angle = 180
                 self.reg_line_item.label.updateTransform()
@@ -786,7 +815,7 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
         return size_data
 
     def update_sizes(self):
-        self.master.prepare_data()
+        self.set_data(self.sparse_to_dense())
         self.update_point_size()
 
     def update_point_size(self):
@@ -919,6 +948,7 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
 
     def update_colors(self, keep_colors=False):
         self.master.update_colors()
+        self.set_data(self.sparse_to_dense())
         self.update_alpha_value(keep_colors)
 
     def update_alpha_value(self, keep_colors=False):
@@ -1003,7 +1033,7 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
 
     def assure_attribute_present(self, attr):
         if self.data is not None and attr not in self.data.domain:
-            self.master.prepare_data()
+            self.set_data(self.sparse_to_dense())
 
     def update_grid(self):
         self.plot_widget.showGrid(x=self.show_grid, y=self.show_grid)
